@@ -1,10 +1,12 @@
+import math
 import traceback
 import threading
-from collections import OrderedDict
+from collections import OrderedDict, defaultdict
 from datetime import timedelta
 from decimal import Decimal
-from typing import Union
+from typing import Optional, Union
 
+import pandas as pd
 import polars as pl
 import pytz
 
@@ -14,9 +16,99 @@ from lumibot.entities import Asset, Order, Position, TradingFee
 from lumibot.tools.lumibot_logger import get_logger
 from lumibot.trading_builtins import CustomStream
 
+try:
+    from lumibot.backtesting.thetadata_backtesting_pandas import ThetaDataBacktestingPandas
+except Exception:  # pragma: no cover - optional dependency
+    ThetaDataBacktestingPandas = None
+
 logger = get_logger(__name__)
 
 
+# Typical initial margin requirements for common futures contracts
+# Used for backtesting to simulate margin deduction/release
+TYPICAL_FUTURES_MARGINS = {
+    # CME Micro E-mini Futures
+    "MES": 1300,      # Micro E-mini S&P 500 (~$1,300)
+    "MNQ": 1700,      # Micro E-mini Nasdaq-100 (~$1,700)
+    "MYM": 1100,      # Micro E-mini Dow (~$1,100)
+    "M2K": 800,       # Micro E-mini Russell 2000 (~$800)
+    "MCL": 1500,      # Micro Crude Oil (~$1,500)
+    "MGC": 1200,      # Micro Gold (~$1,200)
+
+    # CME Standard E-mini Futures
+    "ES": 13000,      # E-mini S&P 500 (~$13,000)
+    "NQ": 17000,      # E-mini Nasdaq-100 (~$17,000)
+    "YM": 11000,      # E-mini Dow (~$11,000)
+    "RTY": 8000,      # E-mini Russell 2000 (~$8,000)
+
+    # CME Full-Size Futures
+    "CL": 8000,       # Crude Oil (~$8,000)
+    "GC": 10000,      # Gold (~$10,000)
+    "SI": 14000,      # Silver (~$14,000)
+    "NG": 3000,       # Natural Gas (~$3,000)
+    "HG": 4000,       # Copper (~$4,000)
+
+    # CME Currency Futures
+    "6E": 2500,       # Euro FX (~$2,500)
+    "6J": 3000,       # Japanese Yen (~$3,000)
+    "6B": 2800,       # British Pound (~$2,800)
+    "6C": 2000,       # Canadian Dollar (~$2,000)
+
+    # CME Interest Rate Futures
+    "ZB": 4000,       # 30-Year T-Bond (~$4,000)
+    "ZN": 2000,       # 10-Year T-Note (~$2,000)
+    "ZF": 1500,       # 5-Year T-Note (~$1,500)
+    "ZT": 800,        # 2-Year T-Note (~$800)
+
+    # CME Agricultural Futures
+    "ZC": 2000,       # Corn (~$2,000)
+    "ZS": 3000,       # Soybeans (~$3,000)
+    "ZW": 2500,       # Wheat (~$2,500)
+    "ZL": 1500,       # Soybean Oil (~$1,500)
+
+    # Default for unknown futures
+    "DEFAULT": 5000,  # Conservative default
+}
+
+
+def get_futures_margin_requirement(asset: Asset) -> float:
+    """
+    Get the initial margin requirement for a futures contract.
+
+    This is used in backtesting to simulate the margin deduction when opening
+    a futures position and margin release when closing.
+
+    Args:
+        asset: The futures Asset object
+
+    Returns:
+        float: Initial margin requirement in dollars
+
+    Note:
+        These are TYPICAL values and may not match current broker requirements.
+        For live trading, brokers handle margin internally.
+    """
+    symbol = asset.symbol.upper()
+
+    # Try exact match first
+    if symbol in TYPICAL_FUTURES_MARGINS:
+        return TYPICAL_FUTURES_MARGINS[symbol]
+
+    # Try base symbol (remove month/year codes like "ESH4" -> "ES")
+    # Most futures symbols are 2-3 characters followed by month/year
+    base_symbol = ''.join(c for c in symbol if c.isalpha())
+    if base_symbol in TYPICAL_FUTURES_MARGINS:
+        return TYPICAL_FUTURES_MARGINS[base_symbol]
+
+    # Unknown contract - use conservative default
+    logger.warning(
+        f"Unknown futures contract '{symbol}'. Using default margin of "
+        f"${TYPICAL_FUTURES_MARGINS['DEFAULT']:.2f}. "
+        f"Consider adding this contract to TYPICAL_FUTURES_MARGINS."
+    )
+    return TYPICAL_FUTURES_MARGINS["DEFAULT"]
+
+
 class BacktestingBroker(Broker):
     # Metainfo
     IS_BACKTESTING_BROKER = True
@@ -53,6 +145,8 @@ class BacktestingBroker(Broker):
         self._last_cache_clear = None
         # Market open lookup cache (populated when calendars are initialized)
         self._market_open_cache = {}
+        # Track per-strategy futures lots for accurate margin/P&L when flipping
+        self._futures_lot_ledgers = defaultdict(list)
     def initialize_market_calendars(self, trading_days_df):
         """Initialize trading calendar and eagerly build caches for backtesting."""
         super().initialize_market_calendars(trading_days_df)
@@ -100,6 +194,35 @@ class BacktestingBroker(Broker):
 
         self._sessions_built = True
 
+    def _ensure_market_open_cache(self):
+        """Populate the market-open cache if it has not been initialized."""
+        if self._market_open_cache or self._trading_days is None:
+            return
+        self._market_open_cache = self._trading_days['market_open'].to_dict()
+
+    def _get_market_open_for_close(self, close_time):
+        self._ensure_market_open_cache()
+        return self._market_open_cache.get(close_time)
+
+    def _contiguous_session_time(self, now, idx):
+        """Return remaining time when sessions share a boundary (e.g., futures, crypto)."""
+        if not self.is_market_open():
+            return None
+
+        next_idx = idx + 1
+        if next_idx >= len(self._trading_days):
+            return None
+
+        next_close = self._trading_days.index[next_idx]
+        next_open = self._get_market_open_for_close(next_close)
+        if next_open is None:
+            return None
+
+        if next_open <= now < next_close:
+            return (next_close - now).total_seconds()
+
+        return None
+
     def _is_market_open_dict(self, now):
         """Fast O(1) day lookup then check few sessions."""
         if not self._sessions_built:
@@ -140,6 +263,8 @@ class BacktestingBroker(Broker):
         tz = self.datetime.tzinfo
         is_pytz = isinstance(tz, (pytz.tzinfo.StaticTzInfo, pytz.tzinfo.DstTzInfo))
 
+        previous_datetime = self.datetime
+
         if isinstance(update_dt, timedelta):
             new_datetime = self.datetime + update_dt
         elif isinstance(update_dt, int) or isinstance(update_dt, float):
@@ -150,6 +275,12 @@ class BacktestingBroker(Broker):
         # This is needed to handle Daylight Savings Time changes
         new_datetime = tz.normalize(new_datetime) if is_pytz else new_datetime
 
+        # Guard against non-advancing timestamps (e.g., DST ambiguity)
+        if new_datetime <= previous_datetime:
+            new_datetime = previous_datetime + timedelta(minutes=1)
+            if is_pytz:
+                new_datetime = tz.normalize(new_datetime)
+
         self.data_source._update_datetime(new_datetime, cash=cash, portfolio_value=portfolio_value)
         if self.option_source:
             self.option_source._update_datetime(new_datetime, cash=cash, portfolio_value=portfolio_value)
@@ -215,17 +346,32 @@ class BacktestingBroker(Broker):
         trading_day = search.iloc[0]
         open_time = trading_day.market_open
 
+        # DEBUG: Log what's happening
+        logger.debug(f"[BROKER DEBUG] get_time_to_open: now={now}, next_trading_day={trading_day.name}, "
+                     f"open_time={open_time}")
+
         # For Backtesting, sometimes the user can just pass in dates (i.e. 2023-08-01) and not datetimes
         # In this case the "now" variable is starting at midnight, so we need to adjust the open_time to be actual
-        # market open time.  In the case where the user passes in a time inside a valid trading day, use that time
+        # market open time.  In the case where the user passes in a valid trading day, use that time
         # as the start of trading instead of market open.
+        # BUT: Only do this if the current day (now.date()) is actually a trading day
         if self.IS_BACKTESTING_BROKER and now > open_time:
-            open_time = self.data_source.datetime_start
+            # Check if now.date() is in trading days before overriding
+            now_date = now.date() if hasattr(now, 'date') else now
+            trading_day_dates = self._trading_days.index.date
+            if now_date in trading_day_dates:
+                logger.debug(f"[BROKER DEBUG] Overriding open_time to datetime_start because now ({now}) is on a "
+                             f"trading day but after market open")
+                open_time = self.data_source.datetime_start
+            else:
+                logger.debug(f"[BROKER DEBUG] NOT overriding open_time because now ({now}) is NOT a trading day")
 
         if now >= open_time:
+            logger.debug(f"[BROKER DEBUG] Market already open: now={now} >= open_time={open_time}, returning 0")
             return 0
 
         delta = open_time - now
+        logger.debug(f"[BROKER DEBUG] Market opens in {delta.total_seconds()} seconds")
         return delta.total_seconds()
 
     def get_time_to_close(self):
@@ -242,11 +388,10 @@ class BacktestingBroker(Broker):
 
         # Directly access the data needed using more efficient methods
         market_close_time = self._trading_days.index[idx]
-        # Use cached dict instead of .at for performance; cache should be ready from initialization
-        if not self._market_open_cache:
-            # Safety: rebuild via centralized path rather than inline logic
-            self.initialize_market_calendars(self._trading_days.reset_index())
-        market_open = self._market_open_cache[market_close_time]
+        market_open = self._get_market_open_for_close(market_close_time)
+        if market_open is None:
+            logger.warning("Missing market_open for %s; cannot compute time_to_close", market_close_time)
+            return None
         market_close = market_close_time  # Assuming this is a scalar value directly from the index
 
         # If we're before the market opens for the found trading day,
@@ -256,30 +401,49 @@ class BacktestingBroker(Broker):
             delta = market_close - now
             return delta.total_seconds()
 
-        delta = market_close - now
-        return delta.total_seconds()
+        delta_seconds = (market_close - now).total_seconds()
+        if delta_seconds <= 0:
+            contiguous_seconds = self._contiguous_session_time(now, idx)
+            if contiguous_seconds is not None:
+                return contiguous_seconds
+
+            logger.debug(
+                "Backtesting clock reached or passed market close (%s >= %s); returning 0 seconds.",
+                now,
+                market_close,
+            )
+            return 0.0
+
+        return delta_seconds
 
     def _await_market_to_open(self, timedelta=None, strategy=None):
         # Process outstanding orders first before waiting for market to open
         # or else they don't get processed until the next day
+        logger.debug(f"[BROKER DEBUG] _await_market_to_open called, current "
+                     f"datetime={self.datetime}, timedelta={timedelta}")
         self.process_pending_orders(strategy=strategy)
 
         time_to_open = self.get_time_to_open()
+        logger.debug(f"[BROKER DEBUG] get_time_to_open returned: {time_to_open}")
 
         # If None is returned, it means we've reached the end of available trading days
         if time_to_open is None:
             logger.info("Backtesting reached end of available trading days data")
+            logger.debug(f"[BROKER DEBUG] time_to_open is None, returning early")
             return
 
         # Allow the caller to specify a buffer (in minutes) before the actual open
         if timedelta:
             time_to_open -= 60 * timedelta
+            logger.debug(f"[BROKER DEBUG] Adjusted time_to_open for timedelta buffer: {time_to_open}")
 
         # Only advance time if there is something positive to advance;
         # prevents zero or negative time updates.
         if time_to_open <= 0:
+            logger.debug(f"[BROKER DEBUG] time_to_open <= 0 ({time_to_open}), returning without advancing time")
             return
 
+        logger.debug(f"[BROKER DEBUG] Advancing time by {time_to_open} seconds")
         self._update_datetime(time_to_open)
 
     def _await_market_to_close(self, timedelta=None, strategy=None):
@@ -499,8 +663,16 @@ class BacktestingBroker(Broker):
         def _cancel_inline(order: Order):
             if order.identifier in canceled_identifiers:
                 return
-            canceled_identifiers.add(order.identifier)
-            self._process_trade_event(order, self.CANCELED_ORDER)
+
+            # BUGFIX: Only process CANCELED event if the order is actually active
+            # Don't try to cancel orders that are already filled or canceled
+            if order.is_active():
+                canceled_identifiers.add(order.identifier)
+                self._process_trade_event(order, self.CANCELED_ORDER)
+            else:
+                logger.debug(f"Order {order.identifier} not active (status={order.status}), skipping cancel event")
+                canceled_identifiers.add(order.identifier)
+
             for child in order.child_orders:
                 _cancel_inline(child)
 
@@ -920,9 +1092,17 @@ class BacktestingBroker(Broker):
         asset_type = getattr(order.asset, "asset_type", None)
         quote_asset_type = getattr(order.quote, "asset_type", None) if hasattr(order, "quote") and order.quote else None
 
+        # For futures, use margin-based cash management (not full notional value)
+        # Futures don't tie up full contract value - only margin requirement
+        if (
+            not is_multileg_parent
+            and asset_type in (Asset.AssetType.FUTURE, Asset.AssetType.CONT_FUTURE)
+        ):
+            self._process_futures_fill(strategy, order, float(price), float(filled_quantity))
+
         # For crypto base with forex quote (like BTC/USD where USD is forex), use cash
         # For crypto base with crypto quote (like BTC/USDT where both are crypto), use positions
-        if (
+        elif (
             not is_multileg_parent
             and asset_type == Asset.AssetType.CRYPTO
             and quote_asset_type == Asset.AssetType.FOREX
@@ -964,18 +1144,93 @@ class BacktestingBroker(Broker):
             self._apply_trade_cost(strategy, trade_cost)
 
     def _process_crypto_quote(self, order, quantity, price):
-        """Override to skip crypto quote processing for crypto+forex trades that are handled with direct cash updates."""
-        # Check if this is a crypto+forex trade
+        """Override to skip quote processing for assets that use direct cash updates or margin-based trading."""
+        # Check asset types
         asset_type = getattr(order.asset, "asset_type", None)
         quote_asset_type = getattr(order.quote, "asset_type", None) if hasattr(order, "quote") and order.quote else None
 
-        # For crypto+forex trades, skip position-based quote processing since we handle cash directly
+        # Skip position-based quote processing for:
+        # 1. Crypto+forex trades (handled with direct cash updates)
+        # 2. Futures contracts (use margin, only realize P&L on close, not full notional)
         if asset_type == Asset.AssetType.CRYPTO and quote_asset_type == Asset.AssetType.FOREX:
             return
 
-        # For crypto+crypto trades, use the original position-based processing
+        if asset_type in (Asset.AssetType.FUTURE, Asset.AssetType.CONT_FUTURE):
+            return
+
+        # For other asset types (crypto+crypto, stocks, etc.), use the original position-based processing
         super()._process_crypto_quote(order, quantity, price)
 
+    def _get_futures_ledger_key(self, strategy, asset):
+        strategy_name = getattr(strategy, "_name", None) or getattr(strategy, "name", "unknown_strategy")
+        asset_symbol = getattr(asset, "symbol", "unknown_asset")
+        asset_type = getattr(asset, "asset_type", "unknown_type")
+        return (strategy_name, asset_symbol, asset_type)
+
+    def _realize_futures_pnl(self, ledger, closing_qty, exit_price, multiplier):
+        remaining = closing_qty
+        realized_pnl = 0.0
+
+        while remaining > 1e-9 and ledger:
+            lot = ledger[0]
+            lot_qty = abs(lot["qty"])
+            take_qty = min(lot_qty, remaining)
+            entry_price = lot["price"]
+
+            if lot["qty"] > 0:
+                realized_pnl += (exit_price - entry_price) * take_qty * multiplier
+                lot["qty"] -= take_qty
+            else:
+                realized_pnl += (entry_price - exit_price) * take_qty * multiplier
+                lot["qty"] += take_qty  # Negative qty for shorts
+
+            if abs(lot["qty"]) < 1e-9:
+                ledger.pop(0)
+
+            remaining -= take_qty
+
+        if remaining > 1e-6:
+            logger.warning("Attempted to close more futures contracts than currently open. Remaining qty: %.6f", remaining)
+
+        return realized_pnl
+
+    def _process_futures_fill(self, strategy, order, price, filled_quantity):
+        multiplier = getattr(order.asset, "multiplier", 1)
+        margin_per_contract = get_futures_margin_requirement(order.asset)
+
+        key = self._get_futures_ledger_key(strategy, order.asset)
+        ledger = self._futures_lot_ledgers[key]
+
+        net_before = sum(lot["qty"] for lot in ledger)
+        signed_fill_qty = filled_quantity if order.is_buy_order() else -filled_quantity
+
+        closing_qty = 0.0
+        if net_before and signed_fill_qty and net_before * signed_fill_qty < 0:
+            closing_qty = min(abs(net_before), filled_quantity)
+
+        opening_qty = filled_quantity - closing_qty
+
+        current_cash = strategy.cash or 0.0
+        new_cash = current_cash
+
+        if closing_qty > 0:
+            realized_pnl = self._realize_futures_pnl(ledger, closing_qty, price, multiplier)
+            new_cash += margin_per_contract * closing_qty
+            new_cash += realized_pnl
+
+        if opening_qty > 0:
+            opening_sign = 1 if signed_fill_qty > 0 else -1
+            ledger.append({
+                "qty": opening_sign * opening_qty,
+                "price": price,
+            })
+            new_cash -= margin_per_contract * opening_qty
+
+        if not ledger:
+            self._futures_lot_ledgers.pop(key, None)
+
+        strategy._set_cash_position(new_cash)
+
     def calculate_trade_cost(self, order: Order, strategy, price: float):
         """Calculate the trade cost of an order for a given strategy"""
         trade_cost = 0
@@ -1153,17 +1408,21 @@ class BacktestingBroker(Broker):
 
             # Get the OHLCV data for the asset if we're using the YAHOO, CCXT data source
             data_source_name = self.data_source.SOURCE.upper()
-            if data_source_name in ["CCXT", "YAHOO", "ALPACA", "DATABENTO"]:
-                # Default to backing up one minute so fills use the next bar, consistent with other sources.
+            if data_source_name in ["CCXT", "YAHOO", "ALPACA", "DATABENTO", "DATABENTO_POLARS"]:
+                # Negative deltas here are intentional: _pull_source_symbol_bars subtracts the offset, so
+                # passing -1 minute yields an effective +1 minute guard that keeps us on the previously
+                # completed bar. See tests/*_lookahead for regression coverage.
                 timeshift = timedelta(minutes=-1)
-                if data_source_name == "DATABENTO":
-                    # DataBento mimics Polygon by requesting two bars to guard against gaps.
+                if data_source_name in {"DATABENTO", "DATABENTO_POLARS"}:
+                    # DataBento feeds can skip minutes around maintenance windows. Giving it a two-minute
+                    # cushion mirrors the legacy Polygon behaviour and avoids falling through gaps.
                     timeshift = timedelta(minutes=-2)
                 elif data_source_name == "YAHOO":
-                    # Yahoo uses day bars; shift one day instead to mirror legacy behavior.
+                    # Yahoo daily bars are stamped at the close (16:00). A one-day backstep keeps fills on
+                    # the previous session so we never peek at the in-progress bar.
                     timeshift = timedelta(days=-1)
                 elif data_source_name == "ALPACA":
-                    # Alpaca minute bars are aligned to the current iteration already.
+                    # Alpaca minute bars line up with our clock already; no offset needed.
                     timeshift = None
 
                 ohlc = self.data_source.get_historical_prices(
@@ -1173,6 +1432,23 @@ class BacktestingBroker(Broker):
                     timeshift=timeshift,
                 )
 
+                if (
+                    ohlc is None
+                    or getattr(ohlc, "df", None) is None
+                    or (hasattr(ohlc.df, "empty") and ohlc.df.empty)
+                ):
+                    if strategy is not None:
+                        display_symbol = getattr(order.asset, "symbol", order.asset)
+                        order_identifier = getattr(order, "identifier", None)
+                        if order_identifier is None:
+                            order_identifier = getattr(order, "id", "<unknown>")
+                        strategy.log_message(
+                            f"[DIAG] No historical bars returned for {display_symbol} at {self.datetime}; "
+                            f"pending {order.order_type} id={order_identifier}",
+                            color="yellow",
+                        )
+                    continue
+
                 # Handle both pandas and polars DataFrames
                 if hasattr(ohlc.df, 'index'):  # pandas
                     dt = ohlc.df.index[-1]
@@ -1206,6 +1482,16 @@ class BacktestingBroker(Broker):
                 )
                 # Check if we got any ohlc data
                 if ohlc is None or ohlc.empty:
+                    if strategy is not None:
+                        display_symbol = getattr(order.asset, "symbol", order.asset)
+                        order_identifier = getattr(order, "identifier", None)
+                        if order_identifier is None:
+                            order_identifier = getattr(order, "id", "<unknown>")
+                        strategy.log_message(
+                            f"[DIAG] No pandas bars for {display_symbol} at {self.datetime}; "
+                            f"canceling {order.order_type} id={order_identifier}",
+                            color="yellow",
+                        )
                     self.cancel_order(order)
                     continue
 
@@ -1293,6 +1579,9 @@ class BacktestingBroker(Broker):
             # Fill the order.
             #############################
 
+            if price is None and self._should_attempt_quote_fallback(open, high, low):
+                price = self._try_fill_with_quote(order, strategy, open, high, low)
+
             # If the price is set, then the order has been filled
             if price is not None:
                 self._execute_filled_order(
@@ -1302,41 +1591,185 @@ class BacktestingBroker(Broker):
                     strategy=strategy,
                 )
             else:
+                if strategy is not None:
+                    display_symbol = getattr(order.asset, "symbol", order.asset)
+                    order_identifier = getattr(order, "identifier", None)
+                    if order_identifier is None:
+                        order_identifier = getattr(order, "id", "<unknown>")
+                    detail = (
+                        f"limit={order.limit_price}, high={high}, low={low}"
+                        if order.order_type == Order.OrderType.LIMIT
+                        else f"type={order.order_type}, high={high}, low={low}, stop={getattr(order, 'stop_price', None)}"
+                    )
+                    strategy.log_message(
+                        f"[DIAG] Order remained open for {display_symbol} ({detail}) "
+                        f"id={order_identifier} at {self.datetime}",
+                        color="yellow",
+                    )
                 continue
 
         # After handling all pending orders, cash settle any residual expired contracts.
         self.process_expired_option_contracts(strategy)
 
+    def _coerce_price(self, value):
+        """Convert numeric inputs to float when possible for safe comparisons."""
+        if value is None:
+            return None
+        try:
+            return float(value)
+        except (TypeError, ValueError):
+            return value
+
+    def _is_invalid_price(self, value):
+        """Determine whether a price is unusable (None or NaN)."""
+        if value is None:
+            return True
+        if isinstance(value, Decimal):
+            try:
+                value = float(value)
+            except (ValueError, TypeError):
+                return True
+        if isinstance(value, float) and math.isnan(value):
+            return True
+        try:
+            if pd.isna(value):
+                return True
+        except Exception:
+            pass
+        return False
+
+    def _bar_has_missing_prices(self, *values) -> bool:
+        return any(self._is_invalid_price(val) for val in values)
+
+    def _should_attempt_quote_fallback(self, open_, high_, low_) -> bool:
+        return self._is_thetadata_source() and self._bar_has_missing_prices(open_, high_, low_)
+
+    def _is_thetadata_source(self) -> bool:
+        if ThetaDataBacktestingPandas is None:
+            return False
+        return isinstance(self.data_source, ThetaDataBacktestingPandas)
+
+    def _get_spread_limit(self, strategy, key: str) -> Optional[float]:
+        if strategy is None or not key:
+            return None
+        if hasattr(strategy, key):
+            try:
+                return float(getattr(strategy, key))
+            except (TypeError, ValueError):
+                return None
+        params = getattr(strategy, "parameters", None)
+        if isinstance(params, dict) and key in params:
+            try:
+                return float(params[key])
+            except (TypeError, ValueError):
+                return None
+        return None
+
+    def _try_fill_with_quote(self, order, strategy, open_=None, high_=None, low_=None) -> Optional[float]:
+        """Attempt to fill an order using ThetaData quotes when OHLC bars are missing."""
+        if not self._is_thetadata_source():
+            return None
+        if not self._bar_has_missing_prices(open_, high_, low_):
+            return None
+        if order.order_type not in (Order.OrderType.LIMIT, Order.OrderType.STOP_LIMIT):
+            return None
+        if not (order.is_buy_order() or order.is_sell_order()):
+            return None
+
+        try:
+            quote = self.get_quote(order.asset, quote=order.quote)
+        except Exception as exc:  # pragma: no cover - defensive log for unexpected broker states
+            self.logger.debug("ThetaData quote lookup failed for %s: %s", getattr(order.asset, "symbol", order.asset), exc)
+            return None
+
+        if quote is None:
+            return None
+
+        bid = self._coerce_price(getattr(quote, "bid", None))
+        ask = self._coerce_price(getattr(quote, "ask", None))
+
+        is_buy = order.is_buy_order()
+        fill_price = ask if is_buy else bid
+        if fill_price is None or self._is_invalid_price(fill_price):
+            return None
+
+        limit_price = self._coerce_price(order.limit_price)
+        if limit_price is not None:
+            if is_buy and fill_price > limit_price:
+                return None
+            if not is_buy and fill_price < limit_price:
+                return None
+
+        spread_key = "max_spread_buy_pct" if is_buy else "max_spread_sell_pct"
+        spread_limit = self._get_spread_limit(strategy, spread_key)
+        if spread_limit is not None and bid is not None and ask is not None:
+            mid = (ask + bid) / 2
+            if mid > 0:
+                spread_pct = (ask - bid) / mid
+                if spread_pct > spread_limit:
+                    if strategy is not None:
+                        strategy.log_message(
+                            f"Skipped ThetaData quote fill for {order.identifier} "
+                            f"(spread {spread_pct:.2%} exceeds {spread_limit:.2%}).",
+                            color="yellow",
+                        )
+                    return None
+
+        if strategy is not None:
+            strategy.log_message(
+                f"Filled {order.identifier} via ThetaData quote @ {fill_price:.2f}.",
+                color="yellow",
+            )
+
+        setattr(order, "_price_source", "quote")
+        return fill_price
+
     def limit_order(self, limit_price, side, open_, high, low):
         """Limit order logic."""
+        open_val = self._coerce_price(open_)
+        high_val = self._coerce_price(high)
+        low_val = self._coerce_price(low)
+        limit_val = self._coerce_price(limit_price)
+
+        if any(self._is_invalid_price(val) for val in (open_val, high_val, low_val, limit_val)):
+            return None
+
         # Gap Up case: Limit wasn't triggered by previous candle but current candle opens higher, fill it now
-        if side == "sell" and limit_price <= open_:
-            return open_
+        if side == "sell" and limit_val <= open_val:
+            return open_val
 
         # Gap Down case: Limit wasn't triggered by previous candle but current candle opens lower, fill it now
-        if side == "buy" and limit_price >= open_:
-            return open_
+        if side == "buy" and limit_val >= open_val:
+            return open_val
 
         # Current candle triggered limit normally
-        if low <= limit_price <= high:
-            return limit_price
+        if low_val <= limit_val <= high_val:
+            return limit_val
 
         # Limit has not been met
         return None
 
     def stop_order(self, stop_price, side, open_, high, low):
         """Stop order logic."""
+        open_val = self._coerce_price(open_)
+        high_val = self._coerce_price(high)
+        low_val = self._coerce_price(low)
+        stop_val = self._coerce_price(stop_price)
+
+        if any(self._is_invalid_price(val) for val in (open_val, high_val, low_val, stop_val)):
+            return None
+
         # Gap Down case: Stop wasn't triggered by previous candle but current candle opens lower, fill it now
-        if side == "sell" and stop_price >= open_:
-            return open_
+        if side == "sell" and stop_val >= open_val:
+            return open_val
 
         # Gap Up case: Stop wasn't triggered by previous candle but current candle opens higher, fill it now
-        if side == "buy" and stop_price <= open_:
-            return open_
+        if side == "buy" and stop_val <= open_val:
+            return open_val
 
         # Current candle triggered stop normally
-        if low <= stop_price <= high:
-            return stop_price
+        if low_val <= stop_val <= high_val:
+            return stop_val
 
         # Stop has not been met
         return None
