@@ -8,7 +8,7 @@ from decimal import Decimal
 
 import pandas_market_calendars as mcal
 from alpaca.trading.client import TradingClient
-from alpaca.trading.enums import QueryOrderStatus
+from alpaca.trading.enums import QueryOrderStatus, PositionSide
 from alpaca.trading.requests import GetOrdersRequest, ReplaceOrderRequest
 from alpaca.trading.stream import TradingStream
 from dateutil import tz
@@ -404,6 +404,13 @@ class Alpaca(Broker):
             avg_fill_price = None
 
         position = Position(strategy, asset, quantity, orders=orders, avg_fill_price=avg_fill_price)
+
+        position.pnl = float(broker_position.unrealized_pl) if broker_position.unrealized_pl else None
+        position.current_price = float(broker_position.current_price) if broker_position.current_price else None
+        position.side = Position.PositionSide.LONG if broker_position.side == PositionSide.LONG else Position.PositionSide.SHORT
+        position.market_value = float(broker_position.market_value) if broker_position.market_value else None
+        
+
         return position
 
     def _pull_broker_position(self, asset):
@@ -555,7 +562,11 @@ class Alpaca(Broker):
 
         # Handle None quantity - skip invalid orders
         if qty_value is None:
-            logger.warning(f"Skipping order {identifier_value} - quantity is None (invalid order data from Alpaca)")
+            logger.warning(
+                f"Skipping order {identifier_value} - quantity is None (invalid order data from Alpaca). "
+                f"Order details: symbol={symbol}, side={side_value}, status={status_value}, "
+                f"order_type={order_type_value}, raw_data={resp_raw}"
+            )
             return None
 
         # Construct Order object
@@ -1170,6 +1181,11 @@ class Alpaca(Broker):
                 strategy_name = strategy.name if strategy else "default"
                 order = self._parse_broker_order(alpaca_order, strategy_name=strategy_name)
 
+                # Skip if parsing returned None (invalid order data)
+                if order is None:
+                    logger.warning(f"OAuth Polling: Skipping invalid order from Alpaca - _parse_broker_order returned None")
+                    continue
+
                 logger.debug(f"OAuth Polling: Processing Alpaca order {order.identifier} with status {order.status}")
 
                 # Check if this order exists in our stored orders
@@ -1279,6 +1295,7 @@ class Alpaca(Broker):
                 raise ValueError(error_msg)
             else:
                 logger.error(f"OAuth Polling error: {e}")
+                logger.error(f"Full traceback: {traceback.format_exc()}")
         # No need to schedule next poll - PollingStream handles this automatically via timeout
 
     def _run_stream(self):
