@@ -6,20 +6,17 @@ from .data_source_backtesting import DataSourceBacktesting
 from .exceptions import NoDataFound, UnavailabeTimestep
 from .interactive_brokers_data import InteractiveBrokersData
 from .pandas_data import PandasData
+from .polars_data import PolarsData
 from .tradier_data import TradierData
-
-from .yahoo_data_polars import YahooDataPolars as YahooData
-
-
-from .polygon_data_polars import PolygonDataPolars as PolygonDataBacktesting
-
 from .bitunix_data import BitunixData
 from .ccxt_backtesting_data import CcxtBacktestingData
 from .example_broker_data import ExampleBrokerData
 from .interactive_brokers_rest_data import InteractiveBrokersRESTData
 from .schwab_data import SchwabData
 from .tradovate_data import TradovateData
+from .yahoo_data import YahooData
 
-from .databento_data_polars_live import DataBentoDataPolarsLive as DataBentoData
-from .databento_data_polars_backtesting import DataBentoDataPolarsBacktesting as DataBentoDataBacktesting
+from .databento_data import DataBentoData, DataBentoDataPandas, DataBentoDataPolars
 from .projectx_data import ProjectXData
+
+from ..backtesting.polygon_backtesting import PolygonDataBacktesting
