@@ -103,8 +103,8 @@ class PandasData(DataSourceBacktesting):
         df = pd.DataFrame(range(len(dt_index)), index=dt_index)
         df = df.sort_index()
 
-        # Create a column for the date portion only
-        df["dates"] = df.index.date
+        # Create a column for the date portion only (normalize to date, keeping as datetime64 type)
+        df["dates"] = df.index.normalize()
 
         # Merge with the trading calendar on the 'dates' column to get market open/close times.
         # Use a left join to keep all rows from the original index.
@@ -145,7 +145,8 @@ class PandasData(DataSourceBacktesting):
 
         else:
             pcal.columns = ["datetime"]
-            pcal["date"] = pcal["datetime"].dt.date
+            # Normalize to date but keep as datetime64 type (not date objects)
+            pcal["date"] = pcal["datetime"].dt.normalize()
             result = pcal.groupby("date").agg(
                 market_open=(
                     "datetime",
@@ -238,6 +239,23 @@ class PandasData(DataSourceBacktesting):
                         logger.info(f"Error getting last price for {tuple_to_find}: price is NaN")
                     return None
 
+                if price is None:
+                    return None
+
+                # Treat non-positive prices as missing data.
+                try:
+                    numeric_price = float(price)
+                except (TypeError, ValueError):
+                    numeric_price = None
+
+                if numeric_price is not None and numeric_price <= 0:
+                    logger.warning(
+                        "Ignoring non-positive price %.4f for %s; treating as missing data.",
+                        numeric_price,
+                        tuple_to_find,
+                    )
+                    return None
+
                 return price
             except Exception as e:
                 logger.info(f"Error getting last price for {tuple_to_find}: {e}")
@@ -282,6 +300,16 @@ class PandasData(DataSourceBacktesting):
                 logger.info(f"Error getting ohlcv_bid_ask for {tuple_to_find}: ohlcv_bid_ask_dict is NaN")
                 return Quote(asset=asset)
 
+            for side_key in ("bid", "ask"):
+                value = ohlcv_bid_ask_dict.get(side_key)
+                if value is not None:
+                    try:
+                        numeric_value = float(value)
+                    except (TypeError, ValueError):
+                        numeric_value = value
+                    if isinstance(numeric_value, (int, float)) and numeric_value <= 0:
+                        ohlcv_bid_ask_dict[side_key] = None
+
             # Convert dictionary to Quote object
             return Quote(
                 asset=asset,
@@ -290,6 +318,8 @@ class PandasData(DataSourceBacktesting):
                 ask=ohlcv_bid_ask_dict.get('ask'),
                 volume=ohlcv_bid_ask_dict.get('volume'),
                 timestamp=dt,
+                bid_size=ohlcv_bid_ask_dict.get('bid_size'),
+                ask_size=ohlcv_bid_ask_dict.get('ask_size'),
                 raw_data=ohlcv_bid_ask_dict
             )
         else:
@@ -301,17 +331,26 @@ class PandasData(DataSourceBacktesting):
             result[asset] = self.get_last_price(asset, quote=quote, exchange=exchange)
         return result
 
-    def find_asset_in_data_store(self, asset, quote=None):
-        if asset in self._data_store:
-            return asset
-        elif quote is not None:
-            asset = (asset, quote)
-            if asset in self._data_store:
-                return asset
-        elif isinstance(asset, Asset) and asset.asset_type in ["option", "future", "stock", "index"]:
-            asset = (asset, Asset("USD", "forex"))
-            if asset in self._data_store:
-                return asset
+    def find_asset_in_data_store(self, asset, quote=None, timestep=None):
+        candidates = []
+
+        if timestep is not None:
+            base_quote = quote if quote is not None else Asset("USD", "forex")
+            candidates.append((asset, base_quote, timestep))
+            if quote is not None:
+                candidates.append((asset, Asset("USD", "forex"), timestep))
+
+        if quote is not None:
+            candidates.append((asset, quote))
+
+        if isinstance(asset, Asset) and asset.asset_type in ["option", "future", "stock", "index"]:
+            candidates.append((asset, Asset("USD", "forex")))
+
+        candidates.append(asset)
+
+        for key in candidates:
+            if key in self._data_store:
+                return key
         return None
 
     def _pull_source_symbol_bars(
@@ -333,7 +372,7 @@ class PandasData(DataSourceBacktesting):
         if not timeshift:
             timeshift = 0
 
-        asset_to_find = self.find_asset_in_data_store(asset, quote)
+        asset_to_find = self.find_asset_in_data_store(asset, quote, timestep)
 
         if asset_to_find in self._data_store:
             data = self._data_store[asset_to_find]
@@ -366,7 +405,7 @@ class PandasData(DataSourceBacktesting):
     ):
         """Pull all bars for an asset"""
         timestep = timestep if timestep else self.MIN_TIMESTEP
-        asset_to_find = self.find_asset_in_data_store(asset, quote)
+        asset_to_find = self.find_asset_in_data_store(asset, quote, timestep)
 
         if asset_to_find in self._data_store:
             data = self._data_store[asset_to_find]
@@ -409,20 +448,24 @@ class PandasData(DataSourceBacktesting):
 
         return result
 
-    def _parse_source_symbol_bars(self, response, asset, quote=None, length=None):
-        """parse broker response for a single asset"""
+    def _parse_source_symbol_bars(self, response, asset, quote=None, length=None, return_polars: bool = False):
+        """parse broker response for a single asset
+
+        CRITICAL: return_polars defaults to False for backwards compatibility.
+        PandasData always returns pandas-backed Bars for consistency.
+        """
         asset1 = asset
         asset2 = quote
         if isinstance(asset, tuple):
             asset1, asset2 = asset
-        bars = Bars(response, self.SOURCE, asset1, quote=asset2, raw=response)
+        bars = Bars(response, self.SOURCE, asset1, quote=asset2, raw=response, return_polars=return_polars)
         return bars
 
     def get_yesterday_dividend(self, asset, quote=None):
-        pass
+        return super().get_yesterday_dividend(asset, quote=quote)
 
     def get_yesterday_dividends(self, assets, quote=None):
-        pass
+        return super().get_yesterday_dividends(assets, quote=quote)
 
     # =======Options methods.=================
     def get_chains(self, asset: Asset, quote: Asset = None, exchange: str = None):
@@ -538,5 +581,5 @@ class PandasData(DataSourceBacktesting):
         elif response is None:
             return None
 
-        bars = self._parse_source_symbol_bars(response, asset, quote=quote, length=length)
+        bars = self._parse_source_symbol_bars(response, asset, quote=quote, length=length, return_polars=return_polars)
         return bars
