@@ -2,6 +2,8 @@ import datetime
 from decimal import Decimal
 from typing import Optional, Union
 
+import numpy as np
+
 import pandas as pd
 
 from lumibot.constants import LUMIBOT_DEFAULT_PYTZ as DEFAULT_PYTZ
@@ -411,6 +413,14 @@ class Data:
 
             length = kwargs.get("length", 1)
             timeshift = kwargs.get("timeshift", 0)
+
+            if isinstance(timeshift, datetime.timedelta):
+                if self.timestep == "day":
+                    timeshift = int(timeshift.total_seconds() / (24 * 3600))
+                else:
+                    timeshift = int(timeshift.total_seconds() / 60)
+                kwargs["timeshift"] = timeshift
+
             data_index = i + 1 - length - timeshift
             is_data = data_index >= 0
             if not is_data:
@@ -437,8 +447,8 @@ class Data:
             The number of periods to get the last price.
         timestep : str
             The frequency of the data to get the last price.
-        timeshift : int
-            The number of periods to shift the data.
+        timeshift : int | datetime.timedelta
+            The number of periods to shift the data, or a timedelta that will be converted to periods.
 
         Returns
         -------
@@ -450,6 +460,49 @@ class Data:
         price = close_price if dt > self.datalines["datetime"].dataline[iter_count] else open_price
         return price
 
+    @check_data
+    def get_price_snapshot(self, dt, length=1, timeshift=0):
+        """Return OHLC, bid/ask, and timestamp metadata for the provided datetime."""
+        iter_count = self.get_iter_count(dt)
+
+        def _get_value(column: str):
+            if column not in self.datalines:
+                return None
+            return self.datalines[column].dataline[iter_count]
+
+        def _get_timestamp(column: str) -> Optional[datetime.datetime]:
+            if column not in self.datalines:
+                return None
+            raw_value = self.datalines[column].dataline[iter_count]
+            if raw_value is None:
+                return None
+            if isinstance(raw_value, float) and np.isnan(raw_value):
+                return None
+            if pd.isna(raw_value):
+                return None
+            if isinstance(raw_value, pd.Timestamp):
+                return raw_value.to_pydatetime()
+            if isinstance(raw_value, datetime.datetime):
+                return raw_value
+            try:
+                ts = pd.Timestamp(raw_value)
+            except Exception:
+                return None
+            return ts.to_pydatetime()
+
+        snapshot = {
+            "open": _get_value("open"),
+            "high": _get_value("high"),
+            "low": _get_value("low"),
+            "close": _get_value("close"),
+            "bid": _get_value("bid"),
+            "ask": _get_value("ask"),
+            "last_trade_time": _get_timestamp("last_trade_time"),
+            "last_bid_time": _get_timestamp("last_bid_time"),
+            "last_ask_time": _get_timestamp("last_ask_time"),
+        }
+        return snapshot
+
     @check_data
     def get_quote(self, dt, length=1, timeshift=0):
         """Returns the last known price of the data.
@@ -462,8 +515,8 @@ class Data:
             The number of periods to get the last price.
         timestep : str
             The frequency of the data to get the last price.
-        timeshift : int
-            The number of periods to shift the data.
+        timeshift : int | datetime.timedelta
+            The number of periods to shift the data, or a timedelta that will be converted to periods.
 
         Returns
         -------
@@ -547,12 +600,27 @@ class Data:
 
         """
 
-        # Get bars.
-        end_row = self.get_iter_count(dt) - timeshift
-        start_row = end_row - length
+        if isinstance(timeshift, datetime.timedelta):
+            if self.timestep == "day":
+                timeshift = int(timeshift.total_seconds() / (24 * 3600))
+            else:
+                timeshift = int(timeshift.total_seconds() / 60)
 
+        end_row = self.get_iter_count(dt) - timeshift
+
+        data_len = len(next(iter(self.datalines.values())).dataline) if self.datalines else 0
+        if end_row > data_len:
+            end_row = data_len
+        if end_row < 0:
+            end_row = 0
+
+        start_row = end_row - length
         if start_row < 0:
             start_row = 0
+        if start_row > end_row:
+            start_row = end_row
+        if start_row == end_row and end_row > 0:
+            start_row = max(0, end_row - 1)
 
         # Cast both start_row and end_row to int
         start_row = int(start_row)
