@@ -13,8 +13,6 @@ import pandas_market_calendars as mcal
 from apscheduler.jobstores.memory import MemoryJobStore
 from apscheduler.schedulers.background import BackgroundScheduler
 from apscheduler.triggers.cron import CronTrigger
-from termcolor import colored
-
 from lumibot.constants import LUMIBOT_DEFAULT_PYTZ
 from lumibot.entities import Asset, Order
 from lumibot.entities import Asset
@@ -100,26 +98,35 @@ class StrategyExecutor(Thread):
                 self._market_type_cache[market_name] = True
                 return True
 
-            # Get market calendar
             cal = mcal.get_calendar(market_name)
 
-            # Test with a recent Monday (typical trading day)
-            test_date = pd.Timestamp('2025-01-13', tz='UTC')  # Monday
-            schedule = cal.schedule(start_date=test_date, end_date=test_date)
+            # Sample ~1.5 weeks so we can observe weekend gaps as well as daily spans.
+            reference_day = pd.Timestamp('2025-01-13', tz='UTC')  # Monday
+            schedule = cal.schedule(
+                start_date=(reference_day - timedelta(days=3)),
+                end_date=(reference_day + timedelta(days=7)),
+            )
 
             if schedule.empty:
-                # No trading on this date, assume it's not continuous
                 self._market_type_cache[market_name] = False
                 return False
 
-            # Calculate trading hours duration
-            market_open = schedule.iloc[0, 0]
-            market_close = schedule.iloc[0, 1]
-            duration_hours = (market_close - market_open).total_seconds() / 3600
+            durations = schedule["market_close"] - schedule["market_open"]
+            avg_duration = durations.mean()
+            duration_hours = avg_duration.total_seconds() / 3600 if avg_duration is not pd.NaT else 0
 
-            # Consider markets with 20+ hours per day as continuous
-            # This catches futures markets that trade ~22-24 hours
-            is_continuous = duration_hours >= 20.0
+            # Detect long breaks (weekends/maintenance) between sessions.
+            if len(schedule) >= 2:
+                next_opens = schedule["market_open"].iloc[1:].reset_index(drop=True)
+                prev_closes = schedule["market_close"].iloc[:-1].reset_index(drop=True)
+                gaps = (next_opens - prev_closes)
+                max_gap = gaps.max()
+                gap_hours = max_gap.total_seconds() / 3600 if isinstance(max_gap, pd.Timedelta) else 0
+            else:
+                gap_hours = 0
+
+            # Treat as continuous only if it runs >=20h *and* has no multi-hour gaps (>=6h) between sessions.
+            is_continuous = (duration_hours >= 20.0) and (gap_hours < 6.0)
 
             self._market_type_cache[market_name] = is_continuous
             return is_continuous
@@ -474,7 +481,7 @@ class StrategyExecutor(Thread):
 
             if (
                 update_cash
-                and asset_type != Asset.AssetType.CRYPTO
+                and asset_type not in (Asset.AssetType.CRYPTO, Asset.AssetType.FUTURE, Asset.AssetType.CONT_FUTURE)
                 and quantity is not None
                 and price is not None
             ):
@@ -509,7 +516,7 @@ class StrategyExecutor(Thread):
 
             if (
                 update_cash
-                and asset_type != Asset.AssetType.CRYPTO
+                and asset_type not in (Asset.AssetType.CRYPTO, Asset.AssetType.FUTURE, Asset.AssetType.CONT_FUTURE)
                 and quantity is not None
                 and price is not None
             ):
@@ -1166,7 +1173,7 @@ class StrategyExecutor(Thread):
                 # For live trading, stop when market closes
                 return False
 
-        self.strategy.log_message(colored(f"Sleeping for {strategy_sleeptime} seconds", color="blue"))
+        self.strategy.logger.debug("Sleeping for %s seconds", strategy_sleeptime)
 
         # Run process orders at the market close time first (if not continuous market)
         if not is_continuous_market:
@@ -1319,6 +1326,17 @@ class StrategyExecutor(Thread):
             self._before_market_opens()
             self.lifecycle_last_date['before_market_opens'] = current_date
 
+    def _ensure_progress_inside_open_session(self, time_to_close):
+        """Advance the broker clock if we're stuck while the market is open."""
+        if self.broker.is_market_open() and (time_to_close is None or time_to_close <= 0):
+            self.strategy.logger.debug(
+                "Broker clock stalled with market open; nudging forward by one second."
+            )
+            self.broker._update_datetime(1)
+            return self.broker.get_time_to_close()
+
+        return time_to_close
+
     def _setup_market_session(self, has_data_source):
         """Set up the market session for non-24/7 markets"""
         # Set date to the start date, but account for minutes_before_opening
@@ -1393,7 +1411,7 @@ class StrategyExecutor(Thread):
             # Set up market session and check if we should continue
             if not self._setup_market_session(has_data_source):
                 return
-            time_to_close = self.broker.get_time_to_close()
+            time_to_close = self._ensure_progress_inside_open_session(self.broker.get_time_to_close())
         else:
             time_to_close = float("inf")
 
