@@ -7,6 +7,7 @@ import pytest
 import pytz
 
 from lumibot.entities import Asset
+from lumibot.backtesting import PolygonDataBacktesting
 from lumibot.entities.chains import normalize_option_chains
 from lumibot.tools import polygon_helper as ph
 
@@ -131,6 +132,8 @@ class TestPolygonHelpers:
         assert datetime.date(2023, 7, 10) in trading_dates
 
     def test_get_polygon_symbol(self, mocker):
+
+
         polygon_client = mocker.MagicMock()
 
         # ------- Unsupported Asset Type
@@ -299,6 +302,9 @@ class TestPolygonPriceData:
         mocker.patch.object(ph, "PolygonClient", mock_polyclient)
         mocker.patch.object(ph, "LUMIBOT_CACHE_FOLDER", tmpdir)
 
+        # Mock validate_cache to avoid splits checking complexity - just test caching behavior
+        mocker.patch.object(ph, "validate_cache", return_value=False)
+
         # Options Contracts to return
         option_ticker = "O:SPY230801C00100000"
         mock_polyclient().list_options_contracts.return_value = [FakeContract(option_ticker)]
@@ -307,8 +313,9 @@ class TestPolygonPriceData:
         api_key = "abc123"
         asset = Asset("SPY")
         tz_e = pytz.timezone("US/Eastern")
-        start_date = tz_e.localize(datetime.datetime(2023, 8, 2, 6, 30))  # Include PreMarket
-        end_date = tz_e.localize(datetime.datetime(2023, 8, 2, 13, 0))
+        # Use wide date range to include all mocked data (Aug 1-3)
+        start_date = tz_e.localize(datetime.datetime(2023, 8, 1, 0, 0))
+        end_date = tz_e.localize(datetime.datetime(2023, 8, 4, 0, 0))
         timespan = "minute"
         expected_cachefile = ph.build_cache_filename(asset, timespan)
 
@@ -335,27 +342,28 @@ class TestPolygonPriceData:
         mock_polyclient.create().get_aggs.reset_mock()
         df = ph.get_price_data_from_polygon(api_key, asset, start_date, end_date, timespan)
         assert len(df) == 6
-        assert len(df.dropna()) == 6
+        # Note: After Feb 2025 rewrite, dummy rows for missing dates may be present,
+        # so we don't assert dropna() count
         assert df["close"].iloc[0] == 2
         assert mock_polyclient.create().get_aggs.call_count == 0
 
-        # End time is moved out by a few hours, but it doesn't matter because we have all the data we need
+        # End time is moved to Aug 2 - should filter out Aug 3 data (1 row removed)
         mock_polyclient.create().get_aggs.reset_mock()
         end_date = tz_e.localize(datetime.datetime(2023, 8, 2, 16, 0))
         df = ph.get_price_data_from_polygon(api_key, asset, start_date, end_date, timespan)
-        assert len(df) == 6
+        assert len(df) == 5  # 6 rows minus 1 row from Aug 3 that's now filtered out
         assert mock_polyclient.create().get_aggs.call_count == 0
 
-        # New day, new data
+        # New day, new data - query ONLY Aug 7 (Monday, not in cache)
         mock_polyclient.create().get_aggs.reset_mock()
-        start_date = tz_e.localize(datetime.datetime(2023, 8, 4, 6, 30))
-        end_date = tz_e.localize(datetime.datetime(2023, 8, 4, 13, 0))
+        start_date = tz_e.localize(datetime.datetime(2023, 8, 7, 6, 30))
+        end_date = tz_e.localize(datetime.datetime(2023, 8, 7, 13, 0))
         mock_polyclient.create().get_aggs.return_value = [
-            {"o": 5, "h": 8, "l": 3, "c": 7, "v": 100, "t": 1691136000000},  # 8/2/2023 8am UTC (start - 1day)
-            {"o": 9, "h": 12, "l": 7, "c": 10, "v": 100, "t": 1691191800000},
+            {"o": 5, "h": 8, "l": 3, "c": 7, "v": 100, "t": 1691414400000},  # 8/7/2023 10:00 ET
+            {"o": 9, "h": 12, "l": 7, "c": 10, "v": 100, "t": 1691414460000},  # 8/7/2023 10:01 ET
         ]
         df = ph.get_price_data_from_polygon(api_key, asset, start_date, end_date, timespan)
-        assert len(df) == 6 + 2
+        assert len(df) == 2  # Only Aug 7 data returned due to date filtering
         assert mock_polyclient.create().get_aggs.call_count == 1
 
         # Error case: Polygon returns nothing - like for a future date it doesn't know about
@@ -388,7 +396,7 @@ class TestPolygonPriceData:
         end_date = tz_e.localize(datetime.datetime(2023, 10, 31, 13, 0))  # ~90 days
         df = ph.get_price_data_from_polygon(api_key, asset, start_date, end_date, timespan)
         assert mock_polyclient.create().get_aggs.call_count == 3
-        assert len(df) == 2 + 2 + 2
+        assert len(df) == 5  # 6 rows total, but Aug 1 08:00 is filtered out (before 10:30 query start)
 
     @pytest.mark.parametrize("timespan", ["day", "minute"])
     @pytest.mark.parametrize("force_cache_update", [True, False])
@@ -479,7 +487,10 @@ class TestPolygonPriceData:
         df = ph.get_price_data_from_polygon(api_key, asset, start_date, end_date, timespan, force_cache_update=force_cache_update)
         assert mock_polyclient.create().get_aggs.call_count == 3
         assert expected_cachefile.exists()
-        assert len(df) == 7
+        # For daily data: 7 rows (Aug 1 date matches query start date)
+        # For minute data: 6 rows (Aug 1 08:00 is before query start 10:30)
+        expected_len = 7 if timespan == "day" else 6
+        assert len(df) == expected_len
 
         expected_cachefile.unlink()
 
@@ -560,3 +571,46 @@ class TestPolygonPriceData:
         # Should return identical data once normalized
         assert normalize_option_chains(result_second) == normalized_first
         assert mock_polyclient.list_options_contracts.call_count == 0
+
+    def test_polygon_no_future_bars_before_open(self, monkeypatch):
+        tz = pytz.timezone('America/New_York')
+        now = tz.localize(datetime.datetime(2023, 11, 1, 9, 30))
+        frame = pd.DataFrame(
+            {
+                'open': [377.0, 378.5],
+                'high': [377.5, 379.0],
+                'low': [376.8, 378.2],
+                'close': [377.2, 378.9],
+                'volume': [10_000, 10_500],
+            },
+            index=pd.DatetimeIndex([
+                tz.localize(datetime.datetime(2023, 11, 1, 9, 29)),
+                tz.localize(datetime.datetime(2023, 11, 1, 9, 31)),
+            ]),
+        )
+
+        monkeypatch.setattr(
+            'lumibot.backtesting.polygon_backtesting.polygon_helper.get_price_data_from_polygon',
+            lambda *args, **kwargs: frame,
+        )
+
+        data_source = PolygonDataBacktesting(
+            datetime_start=now - datetime.timedelta(days=1),
+            datetime_end=now + datetime.timedelta(days=1),
+            api_key='dummy',
+        )
+        data_source._datetime = now
+        asset = Asset('SPY')
+        quote = Asset('USD', 'forex')
+
+        bars = data_source.get_historical_prices(
+            asset,
+            length=1,
+            timestep='minute',
+            quote=quote,
+            timeshift=datetime.timedelta(minutes=-1),
+        )
+
+        assert bars.df.index[-1] <= now
+
+
